package org.ow2.jasmine.probe.jmxconnection.simple;

import java.io.IOException;
import java.net.MalformedURLException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import javax.management.MBeanServerConnection;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;

import org.ow2.jasmine.probe.jmxconnection.api.JmxConnection;
import org.ow2.jasmine.probe.jmxconnection.api.JmxConnectionException;

import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

import org.apache.felix.ipojo.annotations.Component;
import org.apache.felix.ipojo.annotations.Invalidate;
import org.apache.felix.ipojo.annotations.Provides;
import org.apache.felix.ipojo.annotations.Validate;
import org.apache.felix.ipojo.annotations.ServiceProperty;

/**
 * Simple (non pooled) implementation of the JmxConnection service.
 * @author danesa
 *
 */
@Component(name="JmxConnectionService", public_factory=false)
@Provides
public class JmxConnectionImpl implements JmxConnection {


    @ServiceProperty(value="basic")
    private String nature;

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(JmxConnection.class);

    @Validate
    public void start() {
        System.out.println("JmxConnection service activated.");
    }

    @Invalidate
    public void stop() {
        System.out.println("JmxConnection service stopped.");
    }

    /**
     * Create connector for a target.
     * @param targetName name of the target
     * @param targetUrl jmx url for the target
     * @param env properties for connector creation
     * @throws JmxConnectionExceptionThrown if connector could not be created, contains the cause.
     */
    public void createConnector(String targetName, String targetUrl,
            Map<String, Object> env) throws JmxConnectionException {
        JMXServiceURL serviceUrl = null;
        try {
            serviceUrl = new JMXServiceURL(targetUrl);
        } catch (MalformedURLException e) {
            throw new JmxConnectionException(e);
        }
        // Create a connector client for the connector server at the given serviceUrl,
        // and try to connect it.
        JMXConnector connector = null;
        try {
            connector = JMXConnectorFactory.connect(serviceUrl, env);
            logger.info("Connector created for target {0} with url {1}.", targetName, targetUrl);
        } catch (SecurityException e) {
            logger.info("SecurityException raised for target {0} with url {1}.", targetName, targetUrl);
            throw new JmxConnectionException(e);
        } catch (IOException e) {
            // The connector client cannot be made or connection
            // cannot be established because of a communication problem,
            logger.info("IOException raised when trying to connect to target {0} with url {1} ({2}).", targetName, targetUrl, e.getMessage());
            throw new JmxConnectionException(e);
        }
        connectors.put(targetName, connector);
    }

    /**
     * Close the connector with a target.
     * @param target name of the target
     * @exception IOException If the connection cannot be closed cleanly.
     * If this exception is thrown, it is not known whether the server end of the connection has been cleanly closed.
     */
    public void closeConnector(String target) throws IOException {
        if (target == null)  {
            return;
        }
        if (connectors.containsKey(target)) {
            JMXConnector connector = (JMXConnector) connectors.get(target);
            connectors.remove(target);
            try {
                connector.close();
                logger.info("JMX connection clossed with target {0}", target);
            } catch (IOException e) {
                logger.warn("Exception when trying to close JMX connector for target {0} ({1})", e.toString());
                throw e;
            }
        }
    }

    /**
     * Return a an object that implements the MBeanServerConnection interface and allows invoking
     * methods on the target's MBeanServer.
     * The target must have a associated a JMX connector.
     * @param target the target name
     * @return null if target is null or if no connector is available for the target. Otherwise,
     * try to provide a connection object
     * @exception IOException if a valid MBeanServerConnection cannot be provided, for instance because the connector
     *  has been closed, or the connection to the remote MBeanServer has broken.
     */
    public MBeanServerConnection getConnection(String target)
            throws IOException {
        MBeanServerConnection mbsc = null;
        if (target == null)  {
            return null;
        }
        if (connectors.containsKey(target)) {
            JMXConnector connector = (JMXConnector) connectors.get(target);
            mbsc = connector.getMBeanServerConnection();
        }
        return mbsc;
    }

    // -------------------- Private state ---------------------

    /**
     * The map of managed JMX Connectors.
     */
    private Map<String, JMXConnector> connectors = Collections.synchronizedMap(new HashMap <String, JMXConnector>());

}
